#ifndef WORDS__H_
#define WORDS__H_

/**
  * @file WORDS.h
  *
  * Library for working with strings.
  *
  * @author Marcell Dietl, Sven Eric Panitz
  * @date 25.01.2010
  *
  **/

/** Simple implementation of the non-standard boolean. **/
typedef enum {false,true} boolean;

/** String is used for char pointers to increase readability. **/
typedef char* string;

/** Unsigned integer variables are represented as nat. **/
typedef unsigned int nat;

typedef struct {
  /** The number of words stored in the list. **/
  nat length;
  /** The actual array. **/
  string* words;
} WordList;

/**
  * A constructor function for WordList objects.
  *
  * @param length The length of the newly created list.
  * @param words The already initialized array with the words.
  *
  * @result A pointer to the newly created object.
  *
  **/
WordList* newWordList(nat length,string* words);

/**
  * A destructor function for WordList objects. 
  *
  * This functions frees every word stored within the list and
  * furthermore the array, which contained the strings.
  * Finally it frees the WordList object itself.
  *
  * @param this The word list to be deleted.
  *
  **/
void deleteWordList(WordList* this);

/**
  * This function reads in a text file from the filesystem and returns the
  * contents of the file as a single large string in the memory.
  *
  * @param fileName The name of the file to be read.
  *
  * @result The contents of the text file as one large string.
  *
  **/
string readFile(string fileName);

/**
  * Check if a specific character is a vowel.
  *
  * The function checks if a given character is a vowel (upper and lower
  * case letters) and returns the result as boolean.
  *
  * @param c The character to be checked.
  *
  * @result true for a vowel, otherwise false.
  *
  **/
boolean isVowel(char c);

/**
  * Check if a specific character is a german "Umlaut".
  *
  * The function checks if a given character is a german "Umlaut" (upper
  * and lower case letters) and returns the result as boolean.
  *
  * @param c The character to be checked.
  *
  * @result true for a german "Umlaut", otherwise false.
  *
  **/
boolean isUmlaut(char c);

/**
  * This function tests if the given argument is a letter of the roman
  * alphabet or a german "Umlaut" or the german s-z-ligature letter.
  * For the german non-ASCII letters the latin-1 encoding is assumed.
  *
  * @param c The character which is tested to be an alphabetic character.
  *
  * @result true for roman characters, otherwise false.
  *
  **/
boolean isAlpha(char c);

/**
  * Creates a new WordList object from a string by splitting the string into
  * words. The string is splitted whenever a charcter has been found, which is
  * not a letter of the Roman alphabet.
  *
  * @param str The complete text, which has to be splitted into single words.
  *
  * @result The list of words found in the string including duplicates.
  *
  **/
WordList* words(string str);

/**
  * Calculate the length of a string.
  *
  * The function calculates the length of a given string by searching for the
  * appended NULL byte.
  *
  * @param this The string to be analyzed.
  *
  * @result The length of the string this.
  *
  **/
int strLength(string this);

/**
  * Compare two strings with each other to determine equality.
  *
  * The function compares two strings char by char to determine if they
  * are totally equal or not.
  *
  * @param this The first string to be compared with that.
  * @param that The second string to be compared with this.
  *
  * @result true if both strings are equal, false otherwise.
  *
  **/
boolean strEq(string this, string that);

/**
  * Check, whether a string is contained within an array of strings.
  *
  * @param words The array containing strings.
  * @param length The number of elements contained in the array.
  * @param w The string we are looking for in the array.
  *
  * @result true when the word was contained in the array, otherwise false.
  *
  **/
boolean contains(string* words,nat length,string w);

/**
  * A procedure for deleting duplicates from the word list.
  *
  * Internally copies of the strings are being made. The old array
  * and the the old strings get deleted. A new length is calculated
  * and a new array allocated.
  *
  * @param this The word list which gets modified.
  *
  **/ 
void nub(WordList* this);

/**
  * Change a specific character into lower case.
  *
  * The function checks if a given character is a upper case letter and
  * if so, changes it to its lower case equivalent.
  *
  * @param c The character to be checked.
  *
  * @result The lower case equivalent of c.
  *
  **/
char toLower(char c);

/**
  * This procedure modifies every single string stored in the
  * word list into a corresponding string of lower case letters.
  * No new memory is allocated. Nothing gets deleted.
  *
  * @param this The word list which gets modified.
  *
  **/
void wordListToLower(WordList* this);

/**
  * Determine if two strings rhyme with each other.
  *
  * The function checks if string this rhymes with that and vice versa.
  * This is done by analyzing both strings and checking if they are the
  * same - starting from the first vowel of each string.
  *
  * @param this The first string to be compared with that.
  * @param that The second string to be compared with this.
  *
  * @result true if this rhymes with that and vice versa, false otherwise.
  *
  **/
boolean doRhyme(string this, string that);

/**
  * Determine if two strings start with the same consonants.
  *
  * The function checks if string s1 starts with the same consonants
  * as string s2. This is done by analyzing both strings and checking if they
  * are the same - until the first vowel of each string.
  *
  * @param s1 The first string to be compared with s2.
  * @param s2 The second string to be compared with s1.
  *
  * @result true if s1 and s2 start with the same consonants, false otherwise.
  *
  **/
boolean startsWithSameConsonants(string s1, string s2);

#endif
